from numpy import arange, pi, sin as np_sin
from numpy.random import default_rng
from science_signal.signal import Signal


def sin(
    duration: float = 10,
    rate: float = 10,
    frequency: float = 1,
    amplitude: float = 1,
    phase: float = 0,
) -> Signal:
    """
    Generate sinus signal

    duration: duration of the signal in second
    rate: rate of the signal in Hz (s-1)
    frequency: frequency of the wanted sinus
    phase: between 0 and 2 pi (if more, congruate)
    """
    x = arange(0, duration, 1 / rate)
    y = amplitude * np_sin(2 * pi * frequency * x + phase)

    return Signal(
        x,
        y,
    )


def cos(
    duration: float = 10,
    rate: float = 10,
    frequency: float = 1,
    amplitude: float = 1,
    phase: float = 0,
) -> Signal:
    """
    Generate cosinus signal

    duration: duration of the signal in second
    rate: rate of the signal in Hz (s-1)
    frequency: frequency of the wanted sinus
    phase: between 0 and 2 pi (if more, congruate)
    """
    return sin(duration, rate, frequency, amplitude, phase + pi / 2)


def gaussian_noise(
    duration: float = 10,
    rate: float = 10,
    sigma: float = 1,
    mu: float = 0,
    seed: None | int = None,
) -> Signal:
    """
    Generate a gaussian noise signal

    duration: duration of the signal in second
    rate: rate of the signal in hz (s-1)
    sigma: standard deviation of the wanted distribution
    mu: mean of the distribution
    """
    x = arange(0, duration, 1 / rate)
    return Signal(
        x,
        default_rng(seed).normal(mu, sigma, len(x)),
    )
